var React = require('react');

function _extends() {
  _extends = Object.assign || function (target) {
    for (var i = 1; i < arguments.length; i++) {
      var source = arguments[i];

      for (var key in source) {
        if (Object.prototype.hasOwnProperty.call(source, key)) {
          target[key] = source[key];
        }
      }
    }

    return target;
  };

  return _extends.apply(this, arguments);
}

function useService(allMethods) {
  var _React$useState = React.useState({
    isLoading: false,
    isError: null,
    isRefetch: false
  }),
      state = _React$useState[0],
      setState = _React$useState[1];

  var refetch = function refetch() {
    setState(function (prevState) {
      return _extends({}, prevState, {
        isRefetch: Math.random()
      });
    });
  };

  var setLoading = function setLoading(loadingState) {
    setState(function (prevState) {
      return _extends({}, prevState, {
        isLoading: loadingState
      });
    });
  };

  var dataKeyLength = Object.keys(state).length;
  var IS_SERVER = typeof window === 'undefined';
  var useIsomorphicLayoutEffect = IS_SERVER ? React.useEffect : React.useLayoutEffect;

  function _query(key, methodParams, options) {
    if (methodParams === void 0) {
      methodParams = [];
    }

    if (options === void 0) {
      options = {
        withEffect: false,
        hookDeps: []
      };
    }

    var keyString = String(key);
    var effectHooksDeps = options && options.hookDeps ? options.hookDeps : [];
    var serviceFnParams = Array.isArray(methodParams) ? [].concat(methodParams) : [];

    if (options && options.withEffect) {
      if (options.dataAlias) {
        var _ref;

        useIsomorphicLayoutEffect(function () {
          setLoading(true);
          allMethods && allMethods[keyString].apply(allMethods, serviceFnParams).then(function (response) {
            setState(function (prevState) {
              var _extends2;

              return _extends({}, prevState, (_extends2 = {}, _extends2[options.dataAlias] = response, _extends2));
            });
            setLoading(false);
          })["catch"](function (error) {
            setState(function (prevState) {
              return _extends({}, prevState, {
                isError: error
              });
            });
            setLoading(false);
            return Promise.reject(error);
          });
        }, [].concat(effectHooksDeps, [state.isRefetch]));
        return _ref = {}, _ref[options.dataAlias] = state[options.dataAlias], _ref.refetch = refetch, _ref.mutate = function mutate(newValue) {
          return setState(function (prevState) {
            var _extends3;

            return _extends({}, prevState, (_extends3 = {}, _extends3[options.dataAlias] = newValue, _extends3));
          });
        }, _ref;
      } else {
        var dataStateKey = "data" + dataKeyLength;
        useIsomorphicLayoutEffect(function () {
          setLoading(true);
          allMethods && allMethods[keyString].apply(allMethods, serviceFnParams).then(function (response) {
            setState(function (prevState) {
              var _extends4;

              return _extends({}, prevState, (_extends4 = {}, _extends4[dataStateKey] = response, _extends4));
            });
            setLoading(false);
          })["catch"](function (error) {
            setState(function (prevState) {
              return _extends({}, prevState, {
                isError: error
              });
            });
            setLoading(false);
            return Promise.reject(error);
          });
        }, [].concat(effectHooksDeps, [state.isRefetch]));
        return {
          data: state["data" + (dataKeyLength - 1)],
          refetch: refetch,
          mutate: function mutate(value) {
            return setState(function (prevState) {
              var _extends5;

              return _extends({}, prevState, (_extends5 = {}, _extends5["data" + (dataKeyLength - 1)] = value, _extends5));
            });
          }
        };
      }
    } else {
      setLoading(true);
      return allMethods && allMethods[keyString].apply(allMethods, serviceFnParams).then(function (response) {
        setLoading(false);
        return Promise.resolve(response);
      })["catch"](function (error) {
        setState(_extends({}, state, {
          isError: error
        }));
        setLoading(false);
        return Promise.reject(error);
      });
    }
  }

  function queryNonObject(method, methodParams, options) {
    if (methodParams === void 0) {
      methodParams = [];
    }

    if (options === void 0) {
      options = {
        withEffect: false,
        hookDeps: []
      };
    }

    var effectHooksDeps = options && options.hookDeps ? options.hookDeps : [];
    var serviceFnParams = Array.isArray(methodParams) ? [].concat(methodParams) : [];

    if (options && options.withEffect) {
      if (options.dataAlias) {
        var _ref2;

        useIsomorphicLayoutEffect(function () {
          setLoading(true);
          method.apply(void 0, serviceFnParams).then(function (response) {
            setState(function (prevState) {
              var _extends6;

              return _extends({}, prevState, (_extends6 = {}, _extends6[options.dataAlias] = response, _extends6));
            });
            setLoading(false);
          })["catch"](function (error) {
            setState(function (prevState) {
              return _extends({}, prevState, {
                isError: error
              });
            });
            setLoading(false);
            return Promise.reject(error);
          });
        }, [].concat(effectHooksDeps, [state.isRefetch]));
        return _ref2 = {}, _ref2[options.dataAlias] = state[options.dataAlias], _ref2.refetch = refetch, _ref2.mutate = function mutate(newValue) {
          return setState(function (prevState) {
            var _extends7;

            return _extends({}, prevState, (_extends7 = {}, _extends7[options.dataAlias] = newValue, _extends7));
          });
        }, _ref2;
      } else {
        var dataStateKey = "data" + dataKeyLength;
        useIsomorphicLayoutEffect(function () {
          setLoading(true);
          method.apply(void 0, serviceFnParams).then(function (response) {
            setState(function (prevState) {
              var _extends8;

              return _extends({}, prevState, (_extends8 = {}, _extends8[dataStateKey] = response, _extends8));
            });
            setLoading(false);
          })["catch"](function (error) {
            setState(function (prevState) {
              return _extends({}, prevState, {
                isError: error
              });
            });
            setLoading(false);
            return Promise.reject(error);
          });
        }, [].concat(effectHooksDeps, [state.isRefetch]));
        return {
          data: state["data" + (dataKeyLength - 1)],
          refetch: refetch,
          mutate: function mutate(value) {
            return setState(function (prevState) {
              var _extends9;

              return _extends({}, prevState, (_extends9 = {}, _extends9["data" + (dataKeyLength - 1)] = value, _extends9));
            });
          }
        };
      }
    } else {
      setLoading(true);
      return method.apply(void 0, serviceFnParams).then(function (response) {
        setLoading(false);
        return Promise.resolve(response);
      })["catch"](function (error) {
        setState(_extends({}, state, {
          isError: error
        }));
        setLoading(false);
        return Promise.reject(error);
      });
    }
  }

  return {
    query: function query() {
      if (allMethods instanceof Object) {
        return _query;
      } else {
        return queryNonObject;
      }
    },
    isLoading: state.isLoading,
    state: state,
    setServiceState: function setServiceState(key, value) {
      return setState(function (prevState) {
        var _extends10;

        return _extends({}, prevState, (_extends10 = {}, _extends10[key] = value, _extends10));
      });
    }
  };
}

exports.useService = useService;
//# sourceMappingURL=index.js.map
